#include "OurQueue.h"
#include "error.h"

/* Constructor sets the head to NULL - initially, there aren't
 * any elements!
 */
OurQueue::OurQueue() {
	head = NULL;
}

/* Destructor is a canonical "delete a linked list" function. */
OurQueue::~OurQueue() {
	while (head != NULL) {
		Cell* next = head->next;
		delete head;
		head = next;
	}
}

/* To determine the size of the queue, we scan across the nodes
 * and count how many there are.
 */
int OurQueue::size() {
	int result = 0;
	for (Cell* curr = head; curr != NULL; curr = curr->next) {
		result++;
	}
	return result;
}

/* The queue is empty if and only if the head is NULL. */
bool OurQueue::isEmpty() {
	return head == NULL;
}

/* To enqueue a node, we create a new cell, then append it to the list. */
void OurQueue::enqueue(int value) {
	/* Create the new cell. */
	Cell* cell = new Cell;
	cell->value = value;
	cell->next = NULL;
	
	/* Append the cell to the end of the list.  There are two cases here:
	 *
	 * 1. If the head is NULL, then we set the head to be this element.
	 * 2. Otherwise, scan to the end of the list and append the element.
	 */
	if (head == NULL) {
		head = cell;
	} else {
		Cell* curr = head;
		while (curr->next != NULL) {
			curr = curr->next;
		}
		curr->next = cell;
	}
}

/* The front of the queue is the element pointed at by head. */
int OurQueue::front() {
	if (isEmpty()) {
		error("And lo!  The queue was empty.");
	}
	return head->value;
}

/* To dequeue an element, we delete the first linked list cell,
 * then return the value it used to have.
 */
int OurQueue::dequeue() {
	int result = front();
	
	Cell* next = head->next;
	delete head;
	head = next;
	
	return result;
}


