/*******************************************
 * File: linked-lists.cpp
 *
 * A program that shows off a simple use of
 * linked lists.
 */

#include <iostream>
#include <string>
#include "simpio.h"
using namespace std;

/* Type: Cell
 * A cell in a linked list.
 */
struct Cell {
	string value;
	Cell* next;
};

/* Reads a list of values from the user, returning them in a
 * linked list.  The values are returned in the reverse of the
 * order in which they were read.
 */
Cell* readList();

/* Prints out the values in a linked list. */
void printList(Cell* list);

/* Frees all memory from the linked list. */
void deleteList(Cell* list);

/* Prepends the given element to the specified list. */
void listPrepend(Cell*& list, string value) {
	Cell* newCell = new Cell;
	newCell->value = value;
	
	newCell->next = list;
	list = newCell;
}


int main() {
	Cell* list = NULL;
	listPrepend(list, "A");
	listPrepend(list, "B");
	listPrepend(list, "C");
	
	printList(list);
	deleteList(list);
}


void deleteList(Cell* list) {
	while (list != NULL) {
		Cell* next = list->next;
		delete list;
		list = next;
	}
}

void printList(Cell* list) {
	if (list == NULL) return;
	
	cout << list->value << endl;
	printList(list->next);
}

Cell* readList() {
	Cell* result = NULL;

	while (true) {
		string line = getLine("Next entry? ");
		if (line == "") break;
		
		Cell* cell = new Cell;
		cell->value = line;
		
		cell->next = result;
		result = cell;
	}
	
	return result;
}


